package util

import (
	"bytes"
	"github.com/gin-gonic/gin"
	"io"
	"net/http"
	"strings"
)

func Param[T any](c *gin.Context, key string) *T {
	if val, ok := c.Get(key); ok && val != nil {
		i, _ := val.(T)
		return &i
	}
	return nil
}

const (
	ApplicationJSON = "application/json"
	XmlForm         = "application/x-www-form-urlencoded;charset=utf-8"
)

func Get(url string, client http.Client) ([]byte, error) {
	rep, err := client.Get(url)
	if err != nil {
		return nil, err
	}
	defer rep.Body.Close()
	data, err := io.ReadAll(rep.Body)
	if err != nil {
		return nil, err
	}
	return data, nil
}

func PostFormStr(url string, params string, header map[string]string, client http.Client) ([]byte, error) {
	p := StrToUrlValue(params)
	requestStr := p.Encode()
	buf := strings.NewReader(requestStr)
	request, err := http.NewRequest(http.MethodPost, url, buf)
	if err != nil {
		return nil, err
	}
	request.Header.Add("Content-Type", XmlForm)
	if header != nil {
		for k, v := range header {
			request.Header.Add(k, v)
		}
	}

	response, err := client.Do(request)
	if err != nil {
		return nil, err
	}
	defer response.Body.Close()
	data, err := io.ReadAll(response.Body)
	if err != nil {
		return nil, err
	}

	return data, err
}

func PostJSONStr(url string, params string, header map[string]string, client http.Client) ([]byte, error) {
	request, err := http.NewRequest(http.MethodPost, url, bytes.NewBufferString(params))
	if err != nil {
		return nil, err
	}
	request.Header.Add("Content-Type", ApplicationJSON)
	if header != nil {
		for k, v := range header {
			request.Header.Add(k, v)
		}
	}

	response, err := client.Do(request)
	if err != nil {
		return nil, err
	}
	defer response.Body.Close()
	data, err := io.ReadAll(response.Body)
	if err != nil {
		return nil, err
	}

	return data, err
}
